__author__ = "Arijit Basu"
__email__ = "sayanarijit@gmail.com"
__homepage__ = "https://github.com/sayanarijit/moodlmth"
__description__ = "Convert raw HTML pages into python source code"
__version__ = "v0.2.4"
__license__ = "MIT"
__all__ = [
    "__author__",
    "__email__",
    "__homepage__",
    "__description__",
    "__version__",
    "__license__",
    "cli",
    "converter",
]
